<?php
// $Id: ezmlm-threads.php,v 1.2 2002/11/14 21:20:43 evan Exp $
//
// ezmlm-threads.php - ezmlm-php v2.0
// --------------------------------------------------------------
// Builds, maintains & displays thread caches
// These cache files live in $ezmlm->tmpdir and are serialized
// php objects that can be unserialized and displayed easily
// --------------------------------------------------------------

require_once("ezmlm.php");
require_once("ezmlm-parser.php");

// CLASS: ezmlm_threads will build, maintain & display thread files (even if a thread is only 1 msg)
class ezmlm_threads extends ezmlm_php {

	// load: this is the main function that should be called.
	// it first checks to see if the cache files are stale, if they are it calls build
	// other wise it loads them and calls display
	function load($month) {
		if (!is_dir($this->tempdir . "/ezmlm-php-" . $this->listname)) {
			$checksum = $this->tempdir . "/ezmlm-php-" . $this->listname . "-" . $month . "-" . "checksum";
		} else {
			$checksum = $this->tempdir . "/ezmlm-php-" . $this->listname . "/" . $month . "-" . "checksum";
		}
		if (!is_file($checksum)) {
			$this->build($month);
		} else {
			$fd = fopen($checksum,"r");
			while (!preg_match('/^md5:/', $md5)) { $md5 = fgets($fd,4096); }
			fclose($fd);
			$md5 = rtrim(preg_replace('/^md5:/', '', $md5), "\n");
			if ($md5 != $this->md5_of_file($this->listdir . "/archive/threads/" . $month)) {
				print "<!-- $md5 ne " . $this->md5_of_file($this->listdir . "/archive/threads/" . $month) . " -->\n";
				$this->build($month);
			}
		}

		$this->display($month);
	}

	// display: this loads each cache file sequentially and displays the messages in them
	// there is no checking of checksum's done here so load() is the preferred method to
	// view the threads
	function display($month) {
		$seq = 0;
		if (!is_dir($this->tempdir . "/ezmlm-php-" . $this->listname)) {
			$cache = $this->tempdir . "/ezmlm-php-" . $this->listname . "-" . $month;
		} else {
			$cache = $this->tempdir . "/ezmlm-php-" . $this->listname . "/" . $month;
		}

		$months = array(1 => 'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December');
		print "<span class=\"heading\">Threads for " . $months[((int)substr($month,4,2) / 1)] . ", " . substr($month,0,4) . "</span><br><br>\n";
		
		print "<table width=\"100%\" border=0 cellpadding=0 cellspacing=0>\n";
		print "<tr bgcolor=\"" . $this->tablecolours[0] . "\"><td><b>From</b></td><td><b>Subject</b></td><td><b>Date</b></td></tr>\n";
		print "<tr bgcolor=\"" . $this->tablecolours[1] . "\"><td colspan=3><hr></td></tr>\n";
		$ctc = 0;

		if (is_file($cache)) {
			include($cache);
		}
		print "<tr><td colspan=3><hr noshade></td></tr>\n";
                print "</table>\n";
        }


	function thread_to_html($thread) {
		$html = "";
		$lastdepth = -1;
		$thread_curr = $thread;
		while ($thread_curr != NULL) {
			$msg = new ezmlm_parser();
			$msg->parse_file($this->listdir . $thread_curr->file, TRUE);

			$html .= "<tr bgcolor=\"" . $this->tablecolours[$ctc] . "\">\n";

			$html .= "<td>";
			$html .= $this->makelink("action=show_author_msgs&actionargs[]=" . $this->makehash($msg->headers['from']),$this->protect_email($msg->headers['from'],TRUE));
			$html .= "</td>\n";

			$html .= "<td align=left valign=center>";
			$html .= " <a name=\"" . urlencode($thread_curr->file) . "\">";
			for ($i = 0; $i < $thread_curr->depth; $i++) {
				$html .= "&nbsp;&nbsp;";
			}

			if (($this->thread_subjlen > 0) and (strlen($msg->headers['subject']) > $this->thread_subjlen)) {
				$subject = substr($msg->headers['subject'], 0, ($this->thread_subjlen - 3 - ($thread_curr->depth * 2)));
				$subject = $subject . "...";
			} else {
				$subject = $msg->headers['subject'];
			}

			$actionargs = preg_split("/\//", $thread_curr->file);
			$subject = preg_replace("/\[" . $this->listname . "\]/", "", $subject);
			$html .= $this->makelink("action=show_msg&actionargs[]=" . $actionargs[2] . "&actionargs[]=" . $actionargs[3], $subject);
			$html .= "</td>\n";

			$html .= "<td>" . date("M j Y, g:i a", strtotime($msg->headers['date'])) . "</td>\n";
			$html .= "</tr>\n";

			$ctc++;
			if ($ctc == count($this->tablecolours)) { $ctc = 0; }

			$lastdepth = $thread_curr->depth;
			$thread_curr = $thread_curr->next;
		}

		$html .= "<tr><td colspan=3><hr noshade></td></tr>\n";
		return($html);
	}

	// build: takes one argument in the format YYYYMM and builds the thread cache file
	// for that month if the ezmlm thread file exists. The resulting cache file is then
	// stored in $this->tmpdir;
	function build($month) {
		if (!is_file($this->listdir . "/archive/threads/" . $month)) { return FALSE; }

		if (!is_dir($this->tempdir . "/ezmlm-php-" . $this->listname)) {
                        $fd2 = fopen($this->tempdir . "/ezmlm-php-" . $this->listname . "-" . $month,"w+");
                } else {
                        $fd2 = fopen($this->tempdir . "/ezmlm-php-" . $this->listname . "/" . $month,"w+");
                }
		fclose($fd2);

		$fd1 = fopen($this->listdir . "/archive/threads/" . $month, "r");
		while (!feof($fd1)) {
			$line = fgets($fd1,4096);
			if (preg_match('/^[0-9]*\:[a-z]* \[.*/', $line)) {
				// valid ezmlm thread file entry

				$subjectfile = preg_replace("/^[0-9]*\:([a-z]*) \[.*/", "\\1", $line);
                		$subjectfile = substr($subjectfile,0,2) . "/" . substr($subjectfile,2,18);

				$thread_head = NULL;
				$thread_curr = NULL;
				$thread_temp = NULL;
				$thread_depth = 1;

				if (!is_file($this->listdir . "/archive/subjects/" . $subjectfile)) { continue; }
				$fd2 = fopen($this->listdir . "/archive/subjects/" . $subjectfile, "r");
				while (!feof($fd2)) {
					$line2 = fgets($fd2,4096);
					if (preg_match('/^[0-9]/',$line2)) {
						$msgnum = preg_replace('/^([0-9]*):.*/', '\\1', $line2);
						$msgfile = $msgnum % 100;
                                		$msgdir  = (int)($msgnum / 100);
						if ($msgfile < 10) { $msgfile = "0" . $msgfile; }
						$msg = new ezmlm_parser();
						$msg->parse_file($this->listdir . "/archive/" . $msgdir . "/" . $msgfile, TRUE);

						$msgid = $msg->headers['message-id'];
						$inreply = $msg->headers['in-reply-to'];
						$references = $msg->headers['references'];
						$thread_depth = 1;

						if ($thread_head == NULL) {
							$thread_head = new ezmlm_thread(0,"/archive/" . $msgdir . "/" . $msgfile,$msgid);
						} else {
							$thread_curr = new ezmlm_thread($depth,"/archive/" . $msgdir . "/" . $msgfile,$msgid);
							if ($inreply != '') { $thread_curr->inreply = $inreply; }
							if ($references != '') { $thread_curr->references = $references; }
							$thread_head->append($thread_curr);
						}
					}
				}
				fclose($fd2);

				// so now after all that mess $thread_head contains a full thread tree
				// first build the depth of each message based on 'in-reply-to' and 'references'
				unset($thread_temp);
				$thread_temp = NULL;
				$thread_curr =& $thread_head->next;
				while (get_class($thread_curr) == 'ezmlm_thread') {
					unset($thread_temp);
					$thread_temp = NULL;

					if ($thread_curr->inreply != '') { $thread_temp =& $thread_head->find_msgid($thread_curr->inreply); }
					if ($thread_temp == NULL) {
						if ($thread_curr->references != '') {
							$refs = preg_split('/ /', $thread_curr->references);
							$refs = array_pop($refs);
							$thread_temp =& $thread_head->find_msgid($refs);
						}
					}
					if ($thread_temp == NULL) {
						// we couldn't find anything... set depth to 1, the default
						$thread_curr->depth = 1;
					} else {
						// we found a reference, set it to it's depth + 1
						$thread_curr->depth = $thread_temp->depth + 1;
					}

					$thread_curr =& $thread_curr->next;
				}
						

				// now write it to a temp file named MONTH-SEQ where seq is cronologic sequence order of the thread.
				if (!is_dir($this->tempdir . "/ezmlm-php-" . $this->listname)) {
					@mkdir($this->tempdir . "/ezmlm-php-" . $this->listname, 0755);
					if (!is_dir($this->tempdir . "/ezmlm-php-" . $this->listname)) {
						$fd2 = fopen($this->tempdir . "/ezmlm-php-" . $this->listname . "-" . $month, "a");
					} else {
						$fd2 = fopen($this->tempdir . "/ezmlm-php-" . $this->listname . "/" . $month, "a");
					}
				} else {
					$fd2 = fopen($this->tempdir . "/ezmlm-php-" . $this->listname . "/" . $month, "a");
				}

				fputs($fd2,$this->thread_to_html($thread_head));
				fclose($fd2);
			}
		}

		// finally store our checksum
		if (!is_dir($this->tempdir . "/ezmlm-php-" . $this->listname)) {
			$fd2 = fopen($this->tempdir . "/ezmlm-php-" . $this->listname . "-" . $month . "-" . "checksum","w+");
		} else {
			$fd2 = fopen($this->tempdir . "/ezmlm-php-" . $this->listname . "/" . $month . "-" . "checksum","w+");
		}
		fputs($fd2,"md5:" . $this->md5_of_file($this->listdir . "/archive/threads/" . $month) . "\n");
		fclose($fd2);
		fclose($fd1);

		return TRUE;
	}

	// listmessages: prints out a nice little calendar and displays the message
	// totals for each month. The link jumps to the thread listing.
	function listmessages() {
		$dir = opendir($this->listdir . "/archive/threads/");
		while ($item = readdir($dir)) {
			if (($item == ".") or ($item == "..")) { continue; }
			if (preg_match("/^[0-9][0-9][0-9][0-9][0-9][0-9]/",$item)) {
				$fd = fopen($this->listdir . "/archive/threads/" . $item, "r");
				$count = 0;
				while(!feof($fd)) {
					$curthread = fgets($fd,4096);
					$num = preg_replace("/.*\[([0-9].*)\].*/", "\\1", $curthread);
					$count = $count + $num;
				}
				$threadyear = substr($item,0,4);
				$threadmonth = substr($item,4,2);
				$threadcount[$threadyear][$threadmonth] = $count;
			}
		}
		closedir($dir);

		sort($threadcount,SORT_NUMERIC);

		print "<table border=0 cellpadding=4 cellspacing=1>\n";
		print " <tr>\n";
		print "  <td></td><td>Jan</td><td>Feb</td><td>Mar</td><td>Apr</td><td>May</td><td>Jun</td><td>Jul</td><td>Aug</td><td>Sept</td><td>Oct</td><td>Nov</td><td>Dec</td>\n";
		print " </tr>\n";

		while (list($key,$val) = each($threadcount)) {
			print " <tr>\n";
			print "  <td>$key</td>";
			for ($i = 1; $i <= 12; $i++) {
				if ($i < 10) { $key2 = "0" . $i; }
				else { $key2 = $i; }
				if ($threadcount[$key][$key2] > 0) {
					print "<td bgcolor=\"" . $this->tablecolours[0] . "\" align=center valign=middle>";
					print $this->makelink("action=show_threads&actionargs[]=$key$key2",$threadcount[$key][$key2]);
					print "</td>";
				} else {
					print "<td bgcolor=\"" . $this->tablecolours[0] . "\"></td>";
				}
			}
			print "</tr>\n";
		}
		print "</table>\n";
	}
}

// CLASS: ezmlm-thread is a quick little class to allow us to define
// a structure of the current thread in a single-linked list.
// it's a little messy since php doesn't support pointers like C does
// so we have to use references and a head object to append to the list.
class ezmlm_thread {
	var $next;
	var $depth;
	var $file;
	var $msgid;
	var $inreply;
	var $references;
	function append($thread) {
		$thread_curr =& $this;
		while ($thread_curr->next != NULL) {
			$thread_curr =& $thread_curr->next;
		}
		$thread_curr->next = $thread;
	}
	function &find_msgid($msgid) {
		$thread_curr =& $this;
		while ($thread_curr->next != NULL) {
			if (trim($thread_curr->msgid) == trim($msgid)) { return $thread_curr; }
			$thread_curr =& $thread_curr->next;
		}
		return NULL;
	}
	function ezmlm_thread($depth,$file,$msgid) {
		$this->depth = $depth;
		$this->file  = $file;
		$this->msgid = $msgid;
		$this->next = NULL;
	}
}
?>
